<?php
/**
 * UserIdentity represents the data needed to identity a user.
 * It contains the authentication method that checks if the provided
 * data can identity the user.
 */
class UserIdentity extends CUserIdentity
{
	/**
	 * @var array fields to search in Active Directory. All found ones can be accessed using Yii::app()->user->fieldname.
	 * @see authenticate
	 */
	private $_fields = array("samaccountname","mail","memberof","department","displayname","telephonenumber","primarygroupid","objectsid");
	private $_id;
	
	/**
	 * @var array RBAC role mappings based on domain criteria.
	 * @see authenticate
	 */	
	private $_roleMappings = array(
			'domain_user' => array(
				'samaccountname'=> array(),
				'groups'=> array('Domain Users'),
			),
			'administrator' => array(
				'samaccountname'=> array(),
				'groups'=> array('Domain Admins'),
			),
		);
	public $username;
	
	/**
	 * Authenticates a user using Active Directory.
	 * 
	 * @return boolean whether authentication succeeds.
	 */
	public function authenticate()
	{
			// alter search fields if needed
	        if(isset(Yii::app()->params['adFields']))
				  $this->_fields = Yii::app()->params['adFields'];
	        
	        // authenticate
			if (Yii::app()->ldap->authenticate($this->username, $this->password)){
				$this->errorCode = self::ERROR_NONE;
				
				// collect AD info about users
				$info = Yii::app()->ldap->user()->infoCollection($this->username, $this->_fields);
				$groups = Yii::app()->ldap->user()->groups($this->username);
				
				// alter id to domain account name
				$this->_id = $info->samaccountname;
				
				// export ad info to user session
				foreach($this->_fields as $field)
					$this->setState($field, $info->$field);	
				
				// export groups array
				$this->setState('groups', $groups);
				
				// assign roles based on domain data
				$auth=Yii::app()->authManager;
				
				// assign roles
				foreach ($this->_roleMappings as $role => $criteria) {
					// assign per username
					if(in_array(strtolower($this->id), $criteria['samaccountname'])) {
						if(!$auth->isAssigned($role,$this->id)) {
							if($auth->assign($role,$this->id)) {
								Yii::trace("{$this->id} was assigned to role $role based on AD username", 'application.components.UserIdentity');
							}
						}
					}
					
					// assign per group
					if(count(array_intersect($groups, $criteria['groups']))>0) {
						if(!$auth->isAssigned($role,$this->id)) {
							if($auth->assign($role,$this->id)) {
								Yii::trace("{$this->id} was assigned to role $role based on AD groups", 'application.components.UserIdentity');
							}
						}
					}					
				}
				
				Yii::app()->authManager->save();
			}
			else {
				 $this->errorCode = self::ERROR_PASSWORD_INVALID;
			}	        
	     
	        return !$this->errorCode;
	}

    public function getId()
    {
        return $this->_id;
    }
}